MODULE trc
   !!======================================================================
   !!                      ***  MODULE  trc  ***
   !! Passive tracers   :  module for tracers defined
   !!======================================================================
   !! History :
   !!   8.2  !  96-01  (M. Levy)  Original code
   !!        !  99-07  (M. Levy)  for LOBSTER1 or NPZD model
   !!        !  00-04  (O. Aumont, M.A. Foujols)  HAMOCC3 and P3ZD
   !!   9.0  !  04-03  (C. Ethe)  Free form and module
   !!----------------------------------------------------------------------
#if defined key_passivetrc
   !!----------------------------------------------------------------------
   !!   'key_passivetrc'   :                               Passive tracer
   !!---------------------------------------------------------------------
   !!  OPA 9.0 , LODYC-IPSL (2003)
   !!----------------------------------------------------------------------
   !! * Modules used
   USE par_oce
   USE par_trc
   IMPLICIT NONE

   PUBLIC


   !! passive tracers names and units (read in namelist)
   !! --------------------------------------------------
   CHARACTER(len=12), PUBLIC, DIMENSION(jptra) :: &
      ctrcnm  ,   &   !!: tracer name 
      ctrcun          !!: tracer unit

   CHARACTER(len=80), PUBLIC, DIMENSION(jptra) :: &
      ctrcnl          !!: tracer long name 
   
   
   !! parameters for the control of passive tracers
   !! --------------------------------------------------
   INTEGER, PUBLIC ::  &
      numnat          !!: the number of the passive tracer NAMELIST
   
   LOGICAL, PUBLIC, DIMENSION(jptra) ::   &
      lutini          !!:  initialisation from FILE or not (NAMELIST)

   INTEGER , PUBLIC, DIMENSION(jptra) :: &
      nutini          !!: FORTRAN LOGICAL UNIT for initialisation file

   !! passive tracers fields (before,now,after)
   !! --------------------------------------------------
   REAL(wp), PUBLIC  ::  &
      trai            !!: initial total tracer

   REAL(wp), PUBLIC, DIMENSION (jpi,jpj,jpk,jptra) :: &
      trn     ,   &   !!: traceur concentration for actual time step
      tra     ,   &   !!: traceur concentration for next time step
      trb             !!: traceur concentration for before time step


   !! numerical parameter (NAMELIST)
   !! --------------------------------------------------
   REAL(wp), PUBLIC  ::  &
      rsc     ,   &   !!: tuning coefficient for anti-diffusion
      rtrn            !!: value for truncation

   !! namelist parameters
   !! --------------------------------------------------
   INTEGER , PUBLIC  ::  & 
      ncortrc ,   &   !!: number of corrective phases
      ndttrc  ,   &   !!: frequency of step on passive tracers
      nittrc000       !!: first time step of passive tracers model  

   LOGICAL, PUBLIC  ::  & 
      crosster        !!: logical if true computes crossterms


   !! isopycnal scheme for passive tracers
   !! --------------------------------------------------  
   REAL(wp), PUBLIC  ::  &
      ahtrb0  ,   &   !!: background diffusivity coefficient for passive tracer (m2/s)
      trcrat  ,   &   !!: ratio between passive and active tracer coeff for diffusion
      ahtrc0  ,   &   !!: horizontal eddy diffusivity for passive tracers (m2/s)
      aeivtr0         !!: eddy induced velocity coefficient (m2/s)
   
   
   !! passive tracers restart (input and output)
   !! --------------------------------------------------  
   LOGICAL, PUBLIC  ::  &
      lrsttr          !!: boolean term for restart i/o for passive tracers (namelist)
   
   CHARACTER(len=48) , PUBLIC  ::  &
      trestart        !!: name variable of restart file
   
   INTEGER , PUBLIC  ::  &
      nutwrs  ,   &   !!: output FILE for passive tracers restart
      nutrst  ,   &   !!: logical unit for restart FILE for passive tracers
      nrsttr          !!: control of the time step ( 0 or 1 ) for pass. tr.
   
#if defined key_partial_steps
   
   !! interpolated gradient
   !!--------------------------------------------------  
   REAL (wp), PUBLIC, DIMENSION (jpi,jpj,jptra) :: &
      gtru    ,   &   !!: horizontal gradient at u-points at bottom ocean level
      gtrv            !!: horizontal gradient at v-points at bottom ocean level
#else
   REAL (wp), PUBLIC :: &
      gtru    ,   &   !!: horizontal gradient at u-points at bottom ocean level
      gtrv            !!: horizontal gradient at v-points at bottom ocean level
   
#endif
   
#if defined key_trcldf_eiv && defined key_diaeiv
   !! The three component of the eddy induced velocity
   !! --------------------------------------------------
   REAL(wp), PUBLIC, DIMENSION (jpi,jpj,jpk) :: &
      u_trc_eiv,  &   !!: u-eiv (m/s)
      v_trc_eiv,  &   !!: v-eiv (m/s)
      w_trc_eiv       !!: w-eiv (m/s)
#endif
   
   
   !! information for outputs
   !! --------------------------------------------------
   INTEGER , PUBLIC   ::  & 
      nwritetrc,  &   !!: time step frequency for concentration outputs (namelist)
      nit5     ,  &   !!: id for tracer output file
      ndepit5  ,  &   !!: id for depth mesh
      nhorit5  ,  &   !!: id for horizontal mesh
      ndimt50  ,  &   !!: number of ocean points in index array
      ndimt51         !!: number of ocean points in index array
   
   INTEGER , PUBLIC, DIMENSION (jpij)  ::  & 
      ndext51         !!: integer arrays for ocean surface index
   
   INTEGER , PUBLIC, DIMENSION (jpij*jpk)  ::  & 
      ndext50         !!: integer arrays for ocean 3D index
   
   
   !! julian day 
   !! --------------------------------------------------
   INTEGER  , PUBLIC ::  & 
      ijulian         !!: julian day 
   
   REAL(wp), PUBLIC  ::  &
      djulian         !!: julian day ( real)
   
   
# if defined key_trc_diaadd
   !! additional 2D/3D outputs namelist
   !! --------------------------------------------------
   CHARACTER(len=8), PUBLIC, DIMENSION (jpdia2d) ::  & 
      ctrc2d  ,   &   !!: 2d output field name
      ctrc2u          !!: 2d output field unit
   
   CHARACTER(len=8), PUBLIC, DIMENSION (jpdia3d) ::  & 
      ctrc3d ,    &   !!: 3d output field name
      ctrc3u          !!: 3d output field unit
   
   CHARACTER(len=80), PUBLIC, DIMENSION (jpdia2d) ::  & 
      ctrc2l          !!: 2d output field long name
   
   CHARACTER(len=80), PUBLIC, DIMENSION (jpdia3d) ::  & 
      ctrc3l          !!: 3d output field long name
   
   REAL(wp), PUBLIC, DIMENSION (jpi,jpj,jpdia2d) ::  &  
      trc2d           !!:  additional 2d outputs  
   
   REAL(wp), PUBLIC, DIMENSION (jpi,jpj,jpk,jpdia3d) ::  &  
      trc3d           !!:  additional 3d outputs  
   
   
   !! netcdf files and index common
   !! --------------------------------------------------
   INTEGER , PUBLIC :: &
      nwriteadd,  &   !!: frequency of additional arrays outputs(namelist)
      nitd     ,  &   !!: id for additional array output file
      ndepitd  ,  &   !!: id for depth mesh
      nhoritd         !!: id for horizontal mesh
#endif
   
# if defined key_trc_diatrd
   
   !!  non conservative trends (biological, ...)
   !! --------------------------------------------------
   LOGICAL, PUBLIC, DIMENSION (jptra)  ::  &  
      luttrd          !!: large trends diagnostic to write or not (namelist)
   
   !!  dynamical trends
   !!	trtrd()   : trends of the tracer equations
   !!           1 : X advection
   !!           2 : Y advection
   !!           3 : Z advection
   !!           4 : X diffusion
   !!           5 : Y diffusion
   !!           6 : Z diffusion
   !!           7 : X gent velocity
   !!           8 : Y gent velocity
   !!           9 : Z gent velocity
   !! --------------------------------------------------
   
   
   REAL(wp), PUBLIC, DIMENSION(jpi,jpj,jpk,jptra,jpdiatrc) :: &
      trtrd           !!: trends of the tracer equations
   
   !! netcdf files and index common
   !! --------------------------------------------------
   INTEGER , PUBLIC :: &
      nwritetrd       !!: frequency of additional arrays outputs(namelist)
   
   INTEGER , PUBLIC, DIMENSION (jptra)  :: &
      nit6    ,   &   !!: id for additional array output file
      ndepit6 ,   &   !!: id for depth mesh
      nhorit6         !!: id for horizontal mesh
   
#endif 
   
   !! passive tracers data read and at given time_step
   !! --------------------------------------------------
#if defined key_trc_dta
   
   INTEGER , PUBLIC, DIMENSION(jptra) :: &
      numtr   ,   &    !!: logical unit for passive tracers data
      nlectr  ,   &    !!: switch for reading once
      ntrc1   ,   &    !!: ????
      ntrc2            !!: ????
   
   REAL(wp), PUBLIC, DIMENSION (jpi,jpj,jpk,jptra) ::  &  
      trdta            !!: passive tracers data at given time-step
   
   REAL(wp), PUBLIC, DIMENSION (jpi,jpj,jpk,jptra,2) ::  &  
      tracdta          !!: passive tracers data array for two value
                        !: needed for time interpolation
#endif

   !! Contral/debugging
   !! -----------------
   REAL(wp), PUBLIC, DIMENSION(jptra) ::  & 
      tra_ctl         !!: sum of tra trend
#else
   !!======================================================================
   !!  Empty module : No passive tracer 
   !!======================================================================
#endif

END MODULE trc
